#!/usr/bin/env python
# Copyright (c) 2010, 2012  Adam Sloboda

# run with argument 'daemon' to daemonize

# note: needs permissions to read device files (root)
#       requires compatible input-utils (input-kbd) and grep
# bugs: uses magical codes
#       dirty code but works (now even dynamically with udev plug events)

import sys, os # daemonize
import subprocess
from threading import Thread
import struct
import socket

import pyudev
from pyudev.glib import GUDevMonitorObserver
import gobject
# we are using threads
gobject.threads_init()

import alsaaudio
mixer = alsaaudio.Mixer('PCM')

import commands
def output(cmd):
    return commands.getoutput(cmd)

RUNNING = True

# /dev/input/event1
# volume
# type = 1 (EV_KEY)
# code = 114, 115 (down, up)
VOLUME_DOWN = 114
VOLUME_UP = 115
VOLUME_MUTE = 0x71

# /dev/input/event5 (LNXVIDEO)
# brightness down:
BRIGHTNESS_DOWN = 14680065              # tsec
                                        # tusec = 1 (pressed)
# brightness up:
BRIGHTNESS_UP =  14745601               # tsec
                                        # tusec = 1 (pressed)

# OSD
def update(cmd):
    try:
        #sock = socket.socket()
        #sock.connect(('localhost', 12321))
        #sock.send(cmd)
        #sock.close()
        sock = socket.socket(socket.AF_UNIX)
        sock.connect('/tmp/osd.socket')
        sock.send(cmd)
        sock.close()
    except:
        pass

def get_brightness():
    ftmp = open('/sys/class/backlight/acpi_video0/brightness')
    brightness = float(ftmp.read())
    ftmp.close()
    return brightness

# max_brightness
ftmp = open('/sys/class/backlight/acpi_video0/max_brightness')
max_brightness = float(ftmp.read())
ftmp.close()

class Handler(Thread):
    def __init__(self, path, handle):
        Thread.__init__(self)

        # int, int, short, short, int
        self.fmt = 'iiHHi'
        self.f = open(path, 'rb')
        self.handle = handle
        self.path = path

    def run(self):
        print "starting thread"
        self.event = self.f.read(16)

        while self.event and RUNNING:
            if len(self.event) != 16:
                # self.f does not exist anymore
                print 'no event: quitting thread (%s)' % self.path
                return
            self.handle(*struct.unpack(self.fmt, self.event))
            self.event = self.f.read(16)

        self.f.close()
        print "exiting thread"

def handle_vol(tsec, tusec, etype, code, value):
    if etype == 1:
        if code == VOLUME_UP:
            l,r = mixer.getvolume()
            l = min(l+3,100)
            mixer.setvolume(l)
            update('volume %d' % l)

        if code == VOLUME_DOWN:
            l,r = mixer.getvolume()
            l = max(l-6,0)
            mixer.setvolume(l)
            update('volume %d' % l)

        if code == VOLUME_MUTE:
            mixer.setvolume(0)
            update('volume %d' % 0)

#def bin(x, digits=0):
#    oct2bin = ['000','001','010','011','100','101','110','111']
#    binstring = [oct2bin[int(n)] for n in oct(x)]
#    return ''.join(binstring).lstrip('0').zfill(digits)

def handle_lcd(tsec, tusec, etype, code, value):
    #if False:#etype == 0:
    #    #print tsec+tusec/1000000
    #    #print 'type:',bin(etype), hex(etype)
    #    #print 'code:',bin(code), hex(code)
    #    pass
    #else:
    #    print tsec, tusec
    #    print 'type:',bin(etype,16), hex(etype)
    #    print 'code:',bin(code,16), hex(code)
    #    print ''

    # pretty weird format
    #if etype != 0 and tusec == 1 and (tsec == BRIGHTNESS_DOWN or
    #                                  tsec == BRIGHTNESS_UP):
    if etype != 0 and (tsec == BRIGHTNESS_DOWN or
                       tsec == BRIGHTNESS_UP):
        level = get_brightness()
        level = 100 * float(level) / max_brightness
        update('brightness %f' % level)

def has_key(num, key):
    return (0 == os.system('input-kbd %s | grep -q -m1 %s' % (num, key)))

def handle(device):
    name = device.get('NAME')
    if name:
        print name
        path = '/sys' + device.get('DEVPATH') + '/event*/uevent'
        evdev = output('grep DEVNAME %s | cut -d= -f2' % path)
        print evdev
        num = evdev.split('event')[1]

        if has_key(num, 'KEY_BRIGHTNESS'):
            t = Handler('/dev/%s' % evdev, handle_lcd)
            t.daemon = True
            t.start()
        if has_key(num, 'KEY_VOLUME'):
            t = Handler('/dev/%s' % evdev, handle_vol)
            t.daemon = True
            t.start()

def daemonize(stdin='/dev/null', stdout='/dev/null', stderr='/dev/null'):
    # Perform first fork.
    try:
        pid = os.fork()
        if pid > 0:
            sys.exit(0) # Exit first parent.
    except OSError, e:
        sys.stderr.write("fork #1 failed: (%d) %sn" % (e.errno, e.strerror))
        sys.exit(1)
    # Decouple from parent environment.
    os.chdir("/")
    os.umask(0)
    os.setsid()
    # Perform second fork.
    try:
        pid = os.fork()
        if pid > 0:
            sys.exit(0) # Exit second parent.
    except OSError, e:
        sys.stderr.write("fork #2 failed: (%d) %sn" % (e.errno, e.strerror))
        sys.exit(1)
    # The process is now daemonized, redirect standard file descriptors.
    for f in sys.stdout, sys.stderr: f.flush()
    si = file(stdin, 'r')
    so = file(stdout, 'a+')
    se = file(stderr, 'a+', 0)
    os.dup2(si.fileno(), sys.stdin.fileno())
    os.dup2(so.fileno(), sys.stdout.fileno())
    os.dup2(se.fileno(), sys.stderr.fileno())


if __name__ == "__main__":
    if len(sys.argv) > 1 and sys.argv[1] == 'daemon':
        daemonize()

    context = pyudev.Context()

    for device in context.list_devices(subsystem='input'):
        handle(device)

    monitor = pyudev.Monitor.from_netlink(context)
    observer = GUDevMonitorObserver(monitor)
    def callback(mon, dev):
        handle(dev)
    observer.connect('device-added', callback)
    #observer.connect('device-removed', callback)
    monitor.enable_receiving()
    mainloop = gobject.MainLoop()
    mainloop.run()
